from unittest.mock import patch
from gpt_code_review.git_service import GitService

@patch('gpt_code_review.git_service.git.Repo')
def test_init(mock_repo):
    mock_repo.return_value = 'Mock repo'
    service = GitService(repo_path='/path/to/repo', main_branch='main', review_branch='feature')
    assert service.repo == 'Mock repo'
    assert service.main_branch == 'main'
    assert service.review_branch == 'feature'

def test_get_diff():
    mock_commit = {'diff.return_value': ['file1.diff', 'file2.diff']}
    mock_review_branch = {'commit': mock_commit}
    service = GitService(main_branch='main', review_branch=mock_review_branch)
    diffs = service.get_diff()
    assert diffs == ['file1.diff', 'file2.diff']



import git


class GitService:
    def __init__(self, repo_path='.', main_branch='master', review_branch=None, extend=False):
        self.repo = git.Repo(repo_path)
        self.extend = extend
        self.main_branch = self.repo.branches[main_branch]
        self.review_branch = self.repo.branches[review_branch] if review_branch else self.repo.active_branch
        self.diffs = self.get_diff()

    def get_diff(self):
        return self.review_branch.commit.diff(self.get_base_commit())

    def get_base_commit(self):
        review_commit = self.review_branch.commit
        main_commit = self.main_branch.commit
        merge_base = self.repo.merge_base(review_commit, main_commit)
        if not merge_base:
            raise Exception('No merge base found')
        return self.repo.merge_base(review_commit, main_commit)[-1]

    def get_diffrence_per_file(self):
        changes_per_file = {}
        for i in self.get_diff():
            changes_per_file[i.a_path] = self.simple_diff(i.a_path)
        return changes_per_file

    def simple_diff(self, file_path):
        return self.repo.git.diff(self.main_branch, self.repo.active_branch.name, '--', file_path, unified=0,
                                  ignore_blank_lines=True, ignore_space_at_eol=True, ignore_space_change=True,
                                  ignore_all_space=True, ignore_submodules=True, ignore_cr_at_eol=True)

    def preper_message_for_chat(self):
        data = self.get_diffrence_per_file()

        result = """Make code review for this code: \n ```"""
        if self.extend:
            result += f'and make some example how to solve those problems \n'
        for diff in data:
            data[diff] = self.remove_negative_lines(data[diff])
            str_from_diff = f'{diff} \n {data[diff]}'
            result += str_from_diff
        return result + '```'

    def remove_negative_lines(self, data):
        result = []
        splited_str = data.split('\n')
        for line in splited_str:
            if not line.startswith('-'):
                result.append(line)
        return '\n'.join(result)



import os

import click

from .gpt_service import GptService


class SomeFunctionClass:
    @staticmethod
    @click.command()
    @click.option('--gpt_model', default="gpt-3.5-turbo", help='gtp model for generate response')
    @click.option('--token', default=os.getenv('GPT_KEY'), help='You can add it here, or set GPT_KEY evn variable')
    @click.option('--repo_path', default='.', help='Path to repo')
    @click.option('--main_branch', default='master', help='main branch')
    @click.option('--review_branch', default=None, help='branch to compare')
    @click.option('--extend', default=False, help='branch to compare')
    def cli(gpt_model, token, repo_path, main_branch, review_branch, extend):
        service = GptService(gpt_model=gpt_model, token=token, repo_path=repo_path, main_branch=main_branch,
                             review_branch=review_branch, extend=extend)
        print(service.get_message())



from click.testing import CliRunner
from gpt_code_review.main import SomeFunctionClass

def test_cli():
    runner = CliRunner()
    result = runner.invoke(SomeFunctionClass.cli, ['--gpt_model', 'test-model', '--repo_path', '/path/to/repo'])
    assert result.exit_code == 0
    assert 'Generated message from GPT-3' in result.output



from unittest.mock import patch
from gpt_code_review.gpt_service import GptService

@patch('gpt_code_review.gpt_service.openai.ChatCompletion.create')
def test_send_message(mock_create):
    mock_create.return_value = {'choices': [{'message': {'content': 'Generated message from GPT-3'}}]}
    service = GptService()
    message = service.send_message()
    assert message == 'Generated message from GPT-3'

def test_get_message():
    service = GptService()
    service.send_message = lambda: {'choices': [{'message': {'content': 'Generated message from GPT-3'}}]}
    message = service.get_message()
    assert message == 'Generated message from GPT-3'



from setuptools import setup, find_packages

setup(
    name='gpt_code_review',
    version='0.1.3',
    packages=find_packages(),
    author='Patryk Maruda',
    author_email='patrykmaruda@gmail.com',
    description='Code review with GPT-3',
    install_requires=[
        'click',
        'openai',
        'GitPython',
    ],
    entry_points='''
        [console_scripts]
        gpt_code_review=gpt_code_review.main:SomeFunctionClass.cli
    '''
)